TRANSLATOR_GENERIC = 1
TRANSLATOR_BEAUTIFY = 2
TRANSLATOR_MYSQL = 3
TRANSLATOR_SQLITE = 4

PARAM_NONE = 1
PARAM_NUM = 2
PARAM_ASSOC = 3

class Config :

    translator = TRANSLATOR_GENERIC
    binding = PARAM_NUM

    table = ''



from ...Structure import Order
from ...Builder import OrderByBuilder
from typing import Iterable, Mapping

class OrderBy :
    """ORDER BY manipulation component.
    """

    def oderBy(self, columns, orderType) :
        """ORDER BY query manipulation"""
        orderObjects = ()
        if isinstance(columns, str) :
            orderObjects += (Order.create(columns, orderType),)
        elif isinstance(columns, Mapping) :
            for key in columns.keys() :
                orderObjects += (Order.create({key: columns[key]}, orderType),)
        elif isinstance(columns, Iterable) :
            for col in columns :
                orderObjects += (Order.create(col, orderType),)
        for order in orderObjects :
            if isinstance(self.builder, OrderByBuilder) :
                self.builder.addOrder(order)
            else :
                raise Exception('Builder object does not support ORDER BY query')
        return self

    def orderByAsc(self, column) :
        """ORDER BY query with ASC order"""
        return self.oderBy(column, Order.ORDER_ASC)

    def orderByDesc(self, column) :
        """ORDER BY query with DESC order"""
        return self.oderBy(column, Order.ORDER_DESC)



from ...Structure import Column
from ...Builder import GroupByBuilder
from typing import Iterable, Mapping

class GroupBy :
    """GROUP BY manipulation component.
    """

    def groupBy(self, columns) :
        """GROUP BY query manipulation"""
        columnObjects = ()
        if isinstance(columns, str) :
            columnObjects += (Column.create(columns),)
        elif isinstance(columns, Mapping) :
            for key in columns.keys() :
                columnObjects += (Column.create({key: columns[key]}),)
        elif isinstance(columns, Iterable) :
            for col in columns :
                columnObjects += (Column.create(col),)
        for column in columnObjects :
            if isinstance(self.builder, GroupByBuilder) :
                self.builder.addGroup(column)
            else :
                raise Exception('Builder object does not support GROUP BY query')
        return self
